@@ -14,7 +14,8 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-## \addtogroup lal_py_spectrum
+## \defgroup lal_py_spectrum Spectrum
+## \ingroup lal_python
 """This module provides spectral estimation methods, using wrappings
 of the LAL FFT module
 """
@@ -33,6 +34,10 @@ of the LAL FFT module
 # psd = spectrum.median_mean(timeseries, 16384, 8192)
 # \endcode
 
+import warnings
+warnings.warn('lal.spectrum is no longer maintained, and may be removed in a '
+              'future release', DeprecationWarning)
+
 from .. import git_version
 __author__ = "Duncan Macleod <duncan.macleod@ligo.org>"
 __version__ = git_version.id
