@@ -67,13 +67,12 @@
 
 /*---------- empty initializers ---------- */
 
-/*---------- internal Global variables ----------*/
-
+/*---------- internal variables ----------*/
+static FILE *logFile = NULL;
 
 /*---------- internal prototypes ----------*/
 static FILE* LogFile(void);
 
-static const char * LogGetTimestamp (void);
 static const char * LogTimeToString(double t);
 
 static const char *LogFormatLevel( LogLevel_t level );
@@ -94,9 +93,17 @@ LogLevel_t LogLevel(void)
   return LOG_NORMAL;		// Print LOG_CRITICAL and LOG_NORMAL messages by default
 }
 
+/** Set file to print log messages to */
+void LogSetFile(FILE *fp)
+{
+  logFile = fp;
+}
+
 /** Decide where to print log messages */
 static FILE* LogFile(void)
 {
+  if (logFile)
+    return logFile;		// Print all messages to 'logFile' if defined
   if (LogLevel() < LOG_NORMAL)
     return stderr;		// Error log messages are printed to standard error
   return stdout;		// All other log messages are printed to standard output
@@ -194,6 +201,12 @@ XLALGetPeakHeapUsageMB ( void )
   struct rusage usage;
   XLAL_CHECK_REAL8 ( getrusage ( RUSAGE_SELF, &usage ) == 0, XLAL_ESYS, "call to getrusage() failed with errno = %d\n", errno );
   REAL8 peakHeapMB = usage.ru_maxrss / 1024.0;	// maxrss is in KB
+#ifdef __APPLE__
+  // on macOS rusage.ru_maxrss is returned in bytes
+  // https://developer.apple.com/legacy/library/documentation/Darwin/Reference/ManPages/man2/getrusage.2.html
+  peakHeapMB /= 1024.0;
+#endif
+
   if ( lalDebugLevel & LALMEMPADBIT ) {
     peakHeapMB /= 2.0;	// try to correct for memory-padding added by LALMalloc(), which seems ~factor of 2
   }
@@ -250,31 +263,30 @@ XLALGetTimeOfDay ( void )
 /// High-resolution CPU timer (returns result in seconds), aimed for code-timing purposes.
 /// Attempts to provide the highest time resolution available, while adding as little overhead as possible.
 ///
-/// \note uses clock_gettime() with ns precision if available, or falls back to XLALGetTimeOfDay() with
-/// mus-precision otherwise.
+/// \note Returns 0.0 if no CPU timer is available.
 ///
 ///
 REAL8
 XLALGetCPUTime ( void )
 {
-#ifndef HAVE_CLOCK_GETTIME
-  return XLALGetTimeOfDay();
-#else
+#if defined(HAVE_CLOCK_GETTIME) && HAVE_DECL_CLOCK_PROCESS_CPUTIME_ID
 
   struct timespec ut;
-  clockid_t clk_id;
-  clk_id = CLOCK_REALTIME;	// use this as fallback, guaranteed to exist.
-
-  clock_gettime ( clk_id, &ut);	// don't bother testing to avoid overheads, and we would notice in timing if unavailable
-
+  if ( clock_gettime ( CLOCK_PROCESS_CPUTIME_ID, &ut ) != 0 ) {
+    return 0.0;
+  }
   return ut.tv_sec + ut.tv_nsec * 1.e-9;
 
+#else   // no CPU timer available
+
+  return 0.0;
+
 #endif
 } // XLALGetCPUTime()
 
 
 /* returns static timestamps-string for 'now' */
-static const char *
+const char *
 LogGetTimestamp (void)
 {
   return ( LogTimeToString ( XLALGetTimeOfDay() ) );
