@@ -502,6 +502,43 @@ int XLALParseStringValueAsREAL8Range(
 
 }
 
+///
+/// Parse a string representing a range of INT4 values into a INT4Range. Possible formats
+/// are <tt>start</tt>, <tt>start,end</tt>, <tt>start/band</tt>, or <tt>start~plusminus</tt>.
+/// Output range is always <tt>low,high</tt> with <tt>range[0] = low; range[1] = high</tt>.
+///
+int XLALParseStringValueAsINT4Range(
+  INT4Range *int4Range,		///< [out] output range of INT4 values
+  const char *valString			///< [in] input string
+  )
+{
+
+  // Check input
+  XLAL_CHECK(int4Range != NULL, XLAL_EFAULT);
+  XLAL_CHECK(valString != NULL, XLAL_EFAULT);
+  XLAL_CHECK(strlen(valString) > 0, XLAL_EINVAL);
+
+  // Parse range
+  char part[2][256];
+  int T[2][2];
+  XLAL_CHECK( SplitStringIntoRange(valString, part, T) == XLAL_SUCCESS, XLAL_EFUNC );
+  INT4 val[2];
+  XLAL_CHECK( XLALParseStringValueAsINT4(&val[0], part[0]) == XLAL_SUCCESS, XLAL_EFUNC );
+  XLAL_CHECK( XLALParseStringValueAsINT4(&val[1], part[1]) == XLAL_SUCCESS, XLAL_EFUNC );
+  (*int4Range)[0] = T[0][0] * val[0] + T[0][1] * val[1];
+  (*int4Range)[1] = T[1][0] * val[0] + T[1][1] * val[1];
+
+  // Check range ordering
+  if ((*int4Range)[0] > (*int4Range)[1]) {
+    const INT4 tmp = (*int4Range)[0];
+    (*int4Range)[0] = (*int4Range)[1];
+    (*int4Range)[1] = tmp;
+  }
+
+  return XLAL_SUCCESS;
+
+} // XLALParseStringValueAsINT4Range()
+
 
 ///
 /// Parse a string representing a range of LIGOTimeGPS values into a LIGOTimeGPSRange. Possible formats
@@ -617,6 +654,97 @@ int XLALParseStringValueAsDECJRange(
 }
 
 
+///
+/// Parse a string representing a user selection of an enumeration value.
+/// - Enumeration choices with values < 0 are ignored.
+/// - Enumeration choice names are case insensitive.
+///
+int XLALParseStringValueAsUserEnum ( int *valEnum,			///< [out] output enumeration value
+                                     const UserChoices *enumData,	///< [in] possible choices for enumeration values
+                                     const char *valString		///< [in] input string value
+  )
+{
+
+  // Check input
+  XLAL_CHECK(valEnum != NULL, XLAL_EFAULT);
+  XLAL_CHECK(enumData != NULL, XLAL_EFAULT);
+  XLAL_CHECK(valString != NULL, XLAL_EFAULT);
+  XLAL_CHECK(strlen(valString) > 0, XLAL_EINVAL);
+
+  // Select enumeration value
+  for ( size_t i = 0; i < XLAL_NUM_ELEM(*enumData); ++i ) {
+    if ((*enumData)[i].val >= 0 && (*enumData)[i].name != NULL) {
+      if (XLALStringCaseCompare(valString, (*enumData)[i].name) == 0) {
+        *valEnum = (*enumData)[i].val;
+        return XLAL_SUCCESS;
+      }
+    }
+  }
+  XLAL_ERROR( XLAL_EINVAL, "String '%s' does not name a valid enumeration value", valString );
+
+}
+
+
+///
+/// Parse a string representing a user selection of a set of bitflags.
+/// - If the first bitflag choice has a value of zero, it is treated
+//    as a special exclusive case for setting a zero bitflag.
+/// - Bitflag choices with values <= 0 are ignored.
+/// - Bigflag choice names are separated by ',' and are case insensitive.
+///
+int XLALParseStringValueAsUserFlag ( int *valFlag,			///< [out] output bitflag value
+                                     const UserChoices *flagData,	///< [in] possible choices for bitflag values
+                                     const char *valString		///< [in] input string value
+  )
+{
+  LALStringVector *valStringNames = NULL;
+
+  // Check input
+  XLAL_CHECK_FAIL(valFlag != NULL, XLAL_EFAULT);
+  XLAL_CHECK_FAIL(flagData != NULL, XLAL_EFAULT);
+  XLAL_CHECK_FAIL(valString != NULL, XLAL_EFAULT);
+  XLAL_CHECK_FAIL(strlen(valString) > 0, XLAL_EINVAL);
+
+  // Handle special case of first bitflag with value 0, representing a zero bitflag
+  if ((*flagData)[0].val == 0 && (*flagData)[0].name != NULL && XLALStringCaseCompare(valString, (*flagData)[0].name) == 0) {
+    *valFlag = 0;
+    return XLAL_SUCCESS;
+  }
+
+  // Parse input string into bitflag names
+  XLAL_CHECK_FAIL( ( valStringNames = XLALParseStringVector( valString, "," ) ) != NULL, XLAL_EFUNC );
+
+  // Build bitflag value
+  *valFlag = 0;
+  for ( size_t j = 0; j < valStringNames->length; ++j ) {
+    int valFlag_j = 0;
+    for ( size_t i = 0; i < XLAL_NUM_ELEM(*flagData); ++i ) {
+      if ((*flagData)[i].val > 0 && (*flagData)[i].name != NULL) {
+        if (XLALStringCaseCompare(valStringNames->data[j], (*flagData)[i].name) == 0) {
+          valFlag_j = (*flagData)[i].val;
+          break;
+        }
+      }
+    }
+    XLAL_CHECK_FAIL( valFlag_j > 0, XLAL_EINVAL, "String '%s' does not name a valid bitflag value", valStringNames->data[j] );
+    *valFlag |= valFlag_j;
+  }
+
+  // Cleanup
+  XLALDestroyStringVector( valStringNames );
+
+  return XLAL_SUCCESS;
+
+XLAL_FAIL:
+
+  // Cleanup
+  XLALDestroyStringVector( valStringNames );
+
+  return XLAL_FAILURE;
+
+}
+
+
 ///
 /// Duplicate string 'in', removing surrounding quotes \" or \' if present.
 ///
