@@ -294,11 +294,6 @@ do {                                                                 \
     }                                                                \
 } while (0)
 
-/* A global pointer for debugging. */
-#ifndef NDEBUG
-char *lalWatch;
-#endif
-
 int
 main(int argc, char **argv)
 {
